package rs.etf.epos;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.InvalidKeyException;
import java.security.KeyPair;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SignatureException;
import java.security.spec.InvalidKeySpecException;

public interface IDigitalSignatureProvider {

	/**
	 * 
	 * @param signedContent
	 * @param pubKey
	 * @return
	 * @throws InvalidKeyException
	 * @throws SignatureException
	 * @throws NoSuchAlgorithmException 
	 * @throws ClassNotFoundException 
	 * @throws IOException 
	 */
	// boolean verifySignedContent(InputStream signedContent, PublicKey pubKey) throws InvalidKeyException, SignatureException, NoSuchAlgorithmException, IOException, ClassNotFoundException;
	
	
	/**
     * 
     * @param signedContent
     * @param pubKey
     * @return
     * @throws InvalidKeyException
     * @throws SignatureException
	 * @throws ClassNotFoundException 
	 * @throws IOException 
	 * @throws NoSuchAlgorithmException 
     */
    boolean verifySignedContent(byte[] contentToVerify, PublicKey pubKey) throws InvalidKeyException, SignatureException, NoSuchAlgorithmException, IOException, ClassNotFoundException;
    boolean verifySignedContent(byte[] contentToVerify) throws InvalidKeyException, SignatureException, NoSuchAlgorithmException, IOException, ClassNotFoundException, InvalidKeySpecException, NoSuchProviderException;
	
	/**
	 * 
	 * @return
	 */
	public KeyPair generateKeys(String keyType);
	
	/**
	 * 
	 * @param contentToSign
	 * @param signedContentOut
	 * @param privkey
	 * @throws IOException
	 * @throws InvalidKeyException
	 * @throws SignatureException
	 * @throws DigitalSignException 
	 * @throws NoSuchAlgorithmException 
	 */
	void signContent(InputStream contentToSign, OutputStream signedContentOut, PrivateKey privkey) throws IOException, InvalidKeyException, SignatureException, NoSuchAlgorithmException;
	
	/**
     * 
     * @param contentToSign
     * @param signedContentOut
     * @param privkey
	 * @throws IOException 
	 * @throws NoSuchAlgorithmException 
	 * @throws SignatureException 
	 * @throws InvalidKeyException 
     */
    void signContent(byte[] contentToSign, OutputStream signedContentOut, PrivateKey privkey) throws InvalidKeyException, SignatureException, NoSuchAlgorithmException, IOException;
    
    void signContent(byte[] contentToSign, OutputStream signedContentOut, PrivateKey privkey, PublicKey pubkey) throws InvalidKeyException, SignatureException, NoSuchAlgorithmException, IOException;
	
	
	/**
	 * 
	 * @param contentToSign
	 * @param privkey
	 * @return
	 * @throws InvalidKeyException
	 * @throws SignatureException
	 * @throws DigitalSignException 
	 * @throws SignatureException 
	 * @throws NoSuchAlgorithmException 
	 * @throws InvalidKeyException 
	 */
	byte[] generateSignature(byte[] contentToSign, PrivateKey privkey) throws SignatureException, NoSuchAlgorithmException, InvalidKeyException;
	
	
	/**
	 * 
	 * @param signedContent
	 * @param pubkey
	 * @return
	 * @throws ClassNotFoundException 
	 * @throws NoSuchAlgorithmException 
	 * @throws NoSuchProviderException 
	 * @throws InvalidKeySpecException 
	 */
	byte[] extractOriginalContent(byte[] signedContent) throws IOException, NoSuchAlgorithmException, ClassNotFoundException, InvalidKeyException, SignatureException, NoSuchProviderException, InvalidKeySpecException;
	byte[] extractOriginalContent(byte[] signedContent, PublicKey pubkey) throws IOException, NoSuchAlgorithmException, ClassNotFoundException, InvalidKeyException, SignatureException, NoSuchProviderException, InvalidKeySpecException;
	
}
