package rs.etf.epos;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.CipherOutputStream;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.ShortBufferException;

import org.junit.Test;

import static org.junit.Assert.*;

import static rs.etf.epos.Utility.*;

public class CryptoTest {

	static final String ALGORITH = "AES";
	
	void crypt(Cipher cipher, InputStream input, OutputStream out) throws IOException, ShortBufferException, IllegalBlockSizeException, BadPaddingException {
		int blockSize = cipher.getBlockSize();
		byte[] inputBytes = new byte[blockSize];
		CipherOutputStream cos = new CipherOutputStream(out, cipher);
		int inLength = 0;
		while ((inLength = input.read(inputBytes)) != -1) {
			cos.write(inputBytes, 0, inLength);
		}
		cos.flush();
		cos.close();

	}
	
	@Test
	public void testGenerateKey() throws NoSuchAlgorithmException, FileNotFoundException, IOException {
		KeyGenerator keygen = KeyGenerator.getInstance(ALGORITH);
		SecureRandom sr = new SecureRandom();
		keygen.init(sr);
		SecretKey secretKey = keygen.generateKey();
		assertNotNull(secretKey);
		storeObject(new File("crypto.key"), secretKey);
	}
	
	@Test
	public void testEncryption() throws IOException, ClassNotFoundException, NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException, ShortBufferException, IllegalBlockSizeException, BadPaddingException {
		Key key = (Key) loadObjects(new File("crypto.key"), 1)[0];
		Cipher cipher = Cipher.getInstance(ALGORITH);
		cipher.init(Cipher.ENCRYPT_MODE, key);
		
		BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream("encrypted-msg.txt"));
		BufferedInputStream bis = new BufferedInputStream(new FileInputStream("src/message.txt"));
		
		crypt(cipher, bis, bos);
		
		bis.close();
		bos.close();
	}
	
	@Test
	public void testDecryption() throws IOException, ClassNotFoundException, NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException, ShortBufferException, IllegalBlockSizeException, BadPaddingException {
		Key key = (Key) loadObjects(new File("crypto.key"), 1)[0];
		Cipher cipher = Cipher.getInstance(ALGORITH);
		cipher.init(Cipher.DECRYPT_MODE, key);
		
		BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream("decrypted-msg.txt"));
		BufferedInputStream bis = new BufferedInputStream(new FileInputStream("encrypted-msg.txt"));
		
		crypt(cipher, bis, bos);
		
		bis.close();
		bos.close();
	}
	
}
